#include "configfilemanager.h"
#include "historyinfoload.h"
#include <QProcess>

ConfigFileManager::ConfigFileManager(QObject *parent) : QObject(parent)
{

}

/**
 * @brief 创建 JSON 文件
 *
 * 在指定的缓存目录中创建 JSON 文件。
 *
 * @param cachedir 缓存目录对象
 *
 * @return 如果成功创建 JSON 文件，则返回 true；否则返回 false。
 */
bool ConfigFileManager::createJson(QDir cachedir)
{
    QString m_themePath = cachedir.absolutePath();
    QString filePath = m_themePath + "/default.json";

    QDir dir;
    if (!dir.exists(m_themePath)) {
        dir.mkpath(m_themePath);
    }

    if (copyFileContent(":/resource/config/defualt.json", filePath)) {
        // 成功将文件内容写入目标文件中
        return true;
    } else {
        // 写入文件内容失败
        return false;
    }
}

/**
 * @brief 创建配置文件
 *
 * 在给定的缓存目录中创建配置文件。
 *
 * @param cachedir 缓存目录对象
 *
 * @return 创建成功返回 true，否则返回 false
 */
bool ConfigFileManager::createConf(QDir cachedir)
{
    QString m_themePath = cachedir.absolutePath();
    QString filePath = m_themePath + "/theme.conf";

    QDir dir;
    if (!dir.exists(m_themePath)) {
        dir.mkpath(m_themePath);
    }

    if (copyFileContent(":/resource/config/theme.conf", filePath)) {
        // 成功将文件内容写入目标文件中
        return true;
    } else {
        // 写入文件内容失败
        return false;
    }
}

/**
 * @brief 修改radius的json配置文件
 *
 * 根据给定的radius值，修改指定的json配置文件中的"Radius"字段的值。
 *
 * @param radius Radius值
 * @param jsonFilePath 配置文件路径
 *
 * @return 修改成功返回true，否则返回false
 */
bool ConfigFileManager::modifyRadiusJson(int radius,QString jsonFilePath)
{
    QString confFilePath = jsonFilePath;

    // 读取配置文件
    QFile configFile(confFilePath);
    if (!configFile.open(QIODevice::ReadWrite | QIODevice::Text))
    {
        qDebug() << "Failed to open config file"<<configFile.AbortError<<confFilePath;
        return false;
    }

    QByteArray data = configFile.readAll();
    configFile.close();

    // 解析JSON
    QJsonParseError error;
    QJsonDocument jsonDoc = QJsonDocument::fromJson(data, &error);
    if (error.error != QJsonParseError::NoError)
    {
        qDebug() << "Failed to parse JSON:" << error.errorString();
        return false;
    }

    // 修改"value"字段的值为18
    QJsonObject jsonObj = jsonDoc.object();
    QJsonObject radiusObj = jsonObj["Radius"].toObject();
    for (auto it = radiusObj.begin(); it != radiusObj.end(); ++it)
    {
        QJsonObject valueObj = it.value().toObject();
        valueObj["value"] = QString::number(radius);
        radiusObj[it.key()] = valueObj;
    }
    jsonObj["Radius"] = radiusObj;

    // 保存修改后的配置文件
    configFile.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text);
    configFile.write(QJsonDocument(jsonObj).toJson());
    configFile.close();

    qDebug() << "Config file updated successfully";
    emit updateInfo();
}

/**
 * @brief 修改配置文件中的强调色配置
 *
 * 将给定的强调色配置写入指定的配置文件中。
 *
 * @param accentcolor 强调色
 * @param confFilePath 配置文件路径
 *
 * @return 修改成功返回 true，否则返回 false
 */
bool ConfigFileManager::modifyAccentColorConf(QColor accentcolor,QString confFilePath)
{

    QString colorString = accentcolor.name().toUpper();

    while (colorString.length() < 6) {
        colorString.prepend('0');
    }

    // 读取文件内容
    QFile file(confFilePath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << confFilePath;
        return false;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    // 修改内容
    int startIndex = content.indexOf("accent=") + QString("accent=").size();
    content.remove(startIndex, 7);
    content.insert(startIndex, colorString);
    // 写入文件
    if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QTextStream out(&file);
        out << content;
        file.close();
        emit updateInfo();
        return true;
    } else {
        qDebug() << "无法写入文件：" << confFilePath;
        return false;
    }
}

/**
 * @brief 修改配置文件中的强调色配置
 *
 * 将给定的强调色配置写入指定的配置文件中。
 * 注意，modifyQtAccentColorConf用于修改qt主题框架的参数，modifyAccentColorConf用于修改gtk主题的参数
 * @param accentcolor 强调色
 * @param confFilePath 配置文件路径
 *
 * @return 修改成功返回 true，否则返回 false
 */
bool ConfigFileManager::modifyQtAccentColorConf(QString accentcolor, QString confFilePath)
{
    // 将颜色转换为字符串
    QString colorString = accentcolor;

    while (colorString.length() < 6) {
        colorString.prepend('0');
    }

    // 读取文件内容
    QFile file(confFilePath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << confFilePath;
        return false;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    // 修改内容
    int startIndex = content.indexOf("accentRole=") + QString("accentRole=").size();
    int endIndex = content.indexOf("\n", startIndex);
    content.remove(startIndex, endIndex - startIndex);
    content.insert(startIndex, colorString);

    // 写入文件
    if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QTextStream out(&file);
        out << content;
        file.close();
        emit updateInfo();
        return true;
    } else {
        qDebug() << "无法写入文件：" << confFilePath;
        return false;
    }
}

/**
 * @brief 修改壁纸配置文件
 *
 * 从指定的配置文件路径读取文件内容，并修改其中的壁纸路径，然后将修改后的内容写回到文件中。
 *
 * @param wallpaperpath 壁纸路径
 * @param confFilePath 配置文件路径
 *
 * @return 修改成功返回 true，否则返回 false
 */
bool ConfigFileManager::modifyWallPaperConf(QString wallpaperpath,QString confFilePath)
{

    // 读取文件内容
    QFile file(confFilePath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << confFilePath;
        return false;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    // 修改内容
    int startIndex = content.indexOf("wallPaperPath=");
    if (startIndex != -1) {
        QString substr = "wallPaperPath="+wallpaperpath;
        content.replace(startIndex, content.indexOf('\n', startIndex) - startIndex, substr);
    }

    // 写入文件
    if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QTextStream out(&file);
        out << content;
        file.close();
        emit updateInfo();
        return true;
    } else {
        qDebug() << "无法写入文件：" << confFilePath;
        return false;
    }
}

/**
 * @brief 修改透明度配置
 *
 * 读取指定的配置文件内容，并修改其中的透明度设置，最后将修改后的内容写回到文件中。
 *
 * @param transparency 透明度值
 * @param confFilePath 配置文件路径
 *
 * @return 修改成功返回 true，否则返回 false
 */
bool ConfigFileManager::modifyTransparencyConf(int transparency,QString confFilePath)
{
    // 读取文件内容
    QFile file(confFilePath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << confFilePath;
        return false;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    // 修改内容
    QString oldValue = "transparencyBlur=";
    QString newValue = QString("transparencyBlur=%1").arg(transparency);
    content.replace(QRegExp(oldValue + "\\d+"), newValue);

    // 写入文件
    if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QTextStream out(&file);
        out << content;
        file.close();
        emit updateInfo();
        return true;
    } else {
        qDebug() << "无法写入文件：" << confFilePath;
        return false;
    }
}

/**
 * @brief 修改窗口半径配置
 *
 * 将指定的文件中的窗口半径配置进行修改，并保存到文件中。
 *
 * @param windowradius 窗口半径值
 * @param confFilePath 配置文件路径
 *
 * @return 修改成功返回 true，否则返回 false
 */
bool ConfigFileManager::modifywindowRadiusConf(int windowradius,QString confFilePath)
{
    QFile file(confFilePath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Failed to open file: " << confFilePath;
        return false;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    QString oldValue = "windowRadius=";
    QString newValue = QString("windowRadius=%1").arg(windowradius);
    content.replace(QRegExp(oldValue + "\\d+"), newValue);
    if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QTextStream out(&file);
        out << content;
        file.close();
        emit updateInfo();
        return true;
    } else {
        qDebug() << "Failed to write file: " << confFilePath;
        return false;
    }
}

/**
 * @brief 复制文件内容到另一个文件
 *
 * 将源文件的内容复制到目标文件中。
 *
 * @param sourceFilePath 源文件路径
 * @param destinationFilePath 目标文件路径
 *
 * @return 复制成功返回 true，否则返回 false
 */
bool ConfigFileManager::copyFileContent(const QString &sourceFilePath, const QString &destinationFilePath)
{
    QFile sourceFile(sourceFilePath);
    if (!sourceFile.open(QIODevice::ReadOnly)) {
        qDebug() << "Failed to open source file:" << sourceFilePath;
        return false;
    }

    QFile destinationFile(destinationFilePath);
    if (!destinationFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
        qDebug() << "Failed to open destination file:" << destinationFilePath;
        sourceFile.close();
        return false;
    }
    QTextStream in(&sourceFile);
    QTextStream out(&destinationFile);

    while (!in.atEnd()) {
        QString line = in.readLine();
        out << line << "\n";
    }

    sourceFile.close();
    destinationFile.close();

    return true;
}

/**
 * @brief 将文件复制到缓存目录
 *
 * 将指定的文件复制到缓存目录中。如果文件已经存在于缓存目录中，则不会进行复制。
 *
 * @param map 存储键值对的映射
 * @param cachedir 缓存目录对象
 *
 * @return 复制是否成功的布尔值
 */
bool ConfigFileManager::copyIcontoCacheDir(QMap<QString, QString> *map, QDir cachedir)
{
    bool success = true;
    for (auto it = map->begin(); it != map->end(); ++it)
    {
        QString key = it.key();
        QString value = it.value();
//        if (value.startsWith(":/resource")){
//            continue;
//        }

        QFile inputFile(value);
        QFileInfo fileInfo(inputFile.fileName());
        QString destinationPath = cachedir.filePath(key + "." + fileInfo.completeSuffix());

        if(fileInfo.baseName() != key){
            QFile outFile(destinationPath);
            outFile.remove();
        }
        if (!inputFile.exists()){
            success = false;
            continue;
        }

        if (!inputFile.copy(destinationPath)){
            success = false;
        }
    }

    return success;
}

/**
 * @brief 将文件复制到缓存目录
 *
 * 将指定的文件复制到缓存目录中，根据不同的类型设置不同的目标文件名。
 *
 * @param destinationFilePath 目标文件路径
 * @param cachedir 缓存目录
 * @param type 类型
 */
void ConfigFileManager::copyPictoCacheDir(const QString &destinationFilePath, QDir cachedir, QString type)
{
    QString sourceFilePath = destinationFilePath;
    QString sourceFileExt = QFileInfo(sourceFilePath).suffix();

    QString targetFileName = "";
    if(type == "cover"){
        targetFileName = "preview." + sourceFileExt;
    }else if (type == "plymouth"){
        targetFileName = "1." + sourceFileExt;
    }else if (type == "grub"){
        targetFileName = "background." + sourceFileExt;
    }


    QString targetFilePath = cachedir.filePath(targetFileName);

    if (!QFile::copy(sourceFilePath, targetFilePath))
    {
        qDebug() << "Failed to copy file";
        return;
    }
    emit updateInfo();
    qDebug() << "File copied successfully";
}

/**
 * @brief 将文件复制到缓存目录
 *
 * 使用ffmpeg命令将指定的MP4文件复制到缓存目录中，并根据文件名生成相应的缓存文件。
 *
 * @param destinationFilePath 目标文件路径
 * @param cachedir 缓存目录
 */
void ConfigFileManager::copyMp4toCacheDir(const QString &destinationFilePath, QDir cachedir)
{
    QStringList list;
    //ffmpeg -i "111.mp4" "/home/.cache/theme-build/m_time/src/plymouthTheme/%d.png"
    //FIX ME :限制条件
    QString cache = cachedir.absolutePath()+"/%d.png";
    list<<"-i"<<destinationFilePath<<cache;
    QProcess p;
    p.start("ffmpeg",list);
    if (!p.waitForStarted()) {
        qWarning()<<"start get video image failed.";
    }

    if (!p.waitForFinished(40000)) {
        qWarning()<<"wait video image too long time.";
    }
    QString err=p.readAllStandardError();
    QString read=p.readAll();
    if (err.contains("not contain any stream")) {
        qWarning()<<"get video image failed.";
    }
    QString thunbnail = cachedir.absolutePath()+"/40.png";;
    emit updateThumbnail(thunbnail);
}

