#include "historyinfoload.h"
#include <QDebug>
#include <QFileInfo>
#include <QDir>
#include <QFile>
#include <QJsonDocument>
#include <QJsonObject>

/**
 * @brief 构造函数，用于加载历史信息
 *
 * 根据给定的日期，创建一个 HistoryInfoLoad 对象，用于加载历史信息。
 *
 * @param date 日期
 * @param parent 父对象指针
 */
HistoryInfoLoad::HistoryInfoLoad(const QString &date, QObject *parent)
{
    m_historyInfo.historytime = date;
    m_historyInfo.filepath = QDir::homePath() + "/.cache/theme-build/" + m_historyInfo.historytime;
    getWallpaper();
    getAccentColor();
    getRadius();
    getTransparency();
    getWindowRadius();
}

/**
 * @brief 获取封面
 *
 * 从历史信息中获取封面的路径。
 *
 * @return 封面路径
 */
QString HistoryInfoLoad::getCover()
{
    QString filePath = m_historyInfo.filepath + "/src/globalTheme/preview";
    QString previewFilePath;

    QFileInfo pngFileInfo(filePath + ".png");
    if (pngFileInfo.exists()) {
        previewFilePath = pngFileInfo.filePath();
    } else {
        QFileInfo jpgFileInfo(filePath + ".jpg");
        if (jpgFileInfo.exists()) {
            previewFilePath = jpgFileInfo.filePath();
        } else {
            previewFilePath = nullptr;
        }
    }
    m_historyInfo.coverpath = previewFilePath;
    return m_historyInfo.coverpath;
}

/**
 * @brief 获取主题名称
 *
 * 从指定的文件路径中读取主题名称并返回。
 *
 * @return 主题名称
 */
QString HistoryInfoLoad::getThemeName()
{
    QString filePath = m_historyInfo.filepath + "/src/debian/control";

    QFile file(filePath);
    if (file.exists() && file.open(QIODevice::ReadOnly)) {
        QString content(file.readAll());
        file.close();

        QString sourceString = "Source:";
        int sourceIndex = content.indexOf(sourceString);

        if (sourceIndex != -1) {
            int startIndex = sourceIndex + sourceString.length();
            QString fieldValue = content.mid(startIndex).trimmed().split("\n").at(0);

            qDebug() << "Source field value: " << fieldValue;
            m_historyInfo.themename = fieldValue;
        } else {
            qDebug() << "No Source field found.";
        }
    } else {
        qDebug() << "File not found or cannot open file.";
    }
    return m_historyInfo.themename;
}

/**
 * @brief 获取主题类型
 *
 * 根据文件路径获取主题类型。
 *
 * @return 主题类型
 */
QString HistoryInfoLoad::getThemeType()
{
    QString filePath = m_historyInfo.filepath + "/src";
    QDir themeDir(filePath);

    if (themeDir.exists("globalTheme")) {
        m_historyInfo.themetype = "globalTheme";
    } else {
        if (themeDir.exists("cursorTheme")) {
            m_historyInfo.themetype = "cursorTheme";
        } else if (themeDir.exists("iconTheme")) {
            m_historyInfo.themetype = "iconTheme";
        }else if (themeDir.exists("plymouthTheme")) {
            m_historyInfo.themetype = "plymouthTheme";
        }else if (themeDir.exists("grubTheme")) {
            m_historyInfo.themetype = "grubTheme";
        }
    }
    return m_historyInfo.themetype;
}

/**
 * @brief 获取半径
 *
 * 从给定的文件路径中加载半径信息，并保存在 HistoryInfo 对象的 radius 成员变量中。
 */
void HistoryInfoLoad::getRadius()
{
    QString filePath = m_historyInfo.filepath + "/src/globalTheme/default.json";

    QFile file(filePath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Error opening file";
        return;
    }

    QByteArray jsonData = file.readAll();
    file.close();

    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonData);
    if (jsonDoc.isNull()) {
        qDebug() << "jsonDoc is null";
        return;
    }

    QJsonObject rootObj = jsonDoc.object();
    QJsonObject radiusObj = rootObj["Radius"].toObject();
    qDebug() << "Normal_Radius value:" << radiusObj["Normal_Radius"].toObject()["value"];
    qDebug() << "radiusObj:" << radiusObj;
    qDebug() << "Normal_Radius value type:" << radiusObj["Normal_Radius"].toObject()["value"].type();
    QString normalRadiusValue = radiusObj["Normal_Radius"].toObject()["value"].toString();
    m_historyInfo.radius = normalRadiusValue.toInt();

}

/**
 * @brief 获取壁纸
 *
 * 从指定的文件中读取壁纸路径。
 */
void HistoryInfoLoad::getWallpaper()
{
    QString filePath = m_historyInfo.filepath + "/src/globalTheme/theme.conf";
    // 读取文件内容
    QFile file(filePath);
    if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << filePath;
        return;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    // 查找 wallPaperPath= 所在行的位置
    int startIndex = content.indexOf("wallPaperPath=");
    if (startIndex != -1) {
        int lineStartIndex = content.lastIndexOf('\n', startIndex) + 1;
        int lineEndIndex = content.indexOf('\n', startIndex);

        if (lineEndIndex == -1) {
            lineEndIndex = content.size();
        }

        QString lineContent = content.mid(lineStartIndex, lineEndIndex - lineStartIndex).trimmed();
        lineContent.remove("wallPaperPath=");

        m_historyInfo.wallpaperpath = lineContent;
    }

}

/**
 * @brief 获取强调色
 *
 * 从指定路径读取主题配置文件，并获取其中的强调色信息。
 */
void HistoryInfoLoad::getAccentColor()
{
    QString filePath = m_historyInfo.filepath + "/src/globalTheme/theme.conf";
    // 读取文件内容
    QFile file(filePath);
    if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << filePath;
        return;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    int startIndex = content.indexOf("accent=");
    if (startIndex != -1) {
        int lineStartIndex = content.lastIndexOf('\n', startIndex) + 1;
        int lineEndIndex = content.indexOf('\n', startIndex);

        if (lineEndIndex == -1) {
            lineEndIndex = content.size();
        }

        QString lineContent = content.mid(lineStartIndex, lineEndIndex - lineStartIndex).trimmed();
        lineContent.remove("accent=");

        m_historyInfo.accentcolor = lineContent;
    }
}

/**
 * @brief 获取透明度
 *
 * 从指定的路径加载配置文件中获取透明度信息，并将其保存在 HistoryInfo 对象的 transparency 属性中。
 */
void HistoryInfoLoad::getTransparency()
{
    QString filePath = m_historyInfo.filepath + "/src/globalTheme/theme.conf";
    // 读取文件内容
    QFile file(filePath);
    if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << filePath;
        return;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    int startIndex = content.indexOf("transparencyBlur=");
    if (startIndex != -1) {
        int lineStartIndex = content.lastIndexOf('\n', startIndex) + 1;
        int lineEndIndex = content.indexOf('\n', startIndex);

        if (lineEndIndex == -1) {
            lineEndIndex = content.size();
        }

        QString lineContent = content.mid(lineStartIndex, lineEndIndex - lineStartIndex).trimmed();
        lineContent.remove("transparencyBlur=");

        bool conversionSuccess = false;
        int transparencyBlurValue = lineContent.toInt(&conversionSuccess);
        if (conversionSuccess) {
            m_historyInfo.transparency = transparencyBlurValue;
        } else {
            qDebug() << "无法将字符串转换为整数：" << lineContent;
        }
    }
}

/**
 * @brief 获取窗口半径
 *
 * 从指定路径读取主题配置文件，并从中获取窗口半径值。
 */
void HistoryInfoLoad::getWindowRadius()
{
    QString filePath = m_historyInfo.filepath + "/src/globalTheme/theme.conf";
    QFile file(filePath);
    if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << filePath;
        return;
    }

    QTextStream in(&file);
    QString content = in.readAll();
    file.close();

    int startIndex = content.indexOf("windowRadius=");
    if (startIndex != -1) {
        int lineStartIndex = content.lastIndexOf('\n', startIndex) + 1;
        int lineEndIndex = content.indexOf('\n', startIndex);

        if (lineEndIndex == -1) {
            lineEndIndex = content.size();
        }

        QString lineContent = content.mid(lineStartIndex, lineEndIndex - lineStartIndex).trimmed();
        lineContent.remove("windowRadius=");

        bool conversionSuccess = false;
        int windowRadiusValue = lineContent.toInt(&conversionSuccess);
        if (conversionSuccess) {
            m_historyInfo.windowradius = windowRadiusValue;
            qDebug() << "m_historyInfo.windowradius" << windowRadiusValue;
        } else {
            qDebug() << "无法将字符串转换为整数：" << lineContent;
        }
    }
}

/**
 * @brief 获取历史信息数据
 *
 * 返回包含历史信息的数据对象。
 *
 * @return 历史信息数据对象
 */
HistoryInfo HistoryInfoLoad::getInfoData()
{
    qDebug()<<m_historyInfo.accentcolor<<m_historyInfo.wallpaperpath<<m_historyInfo.filepath<<m_historyInfo.windowradius;
    return m_historyInfo;
}

/**
 * @brief 更新历史信息
 *
 * 更新历史信息的函数。
 */
void HistoryInfoLoad::updateHistoryInfo()
{
    getCover();
    getThemeName();
    getThemeType();
    getRadius();
    getWallpaper();
    getAccentColor();
    getTransparency();
    getWindowRadius();
}

