#include "grubthemewidget.h"
#include "../fileProcess/filecheck.h"
#include <QFileDialog>

bool GrubThemeWidget::g_themeChange;
/**
 * @brief GrubThemeWidget 构造函数
 *
 * 创建一个 GrubThemeWidget 对象。
 *
 * @param parent 父对象
 */
GrubThemeWidget::GrubThemeWidget(QWidget *parent) : QWidget(parent)
{
    initPreviewWidget();
    initRightWidget();
    QHBoxLayout *globalthemelayout = new QHBoxLayout();
    globalthemelayout->addWidget(m_previewwidget);
    globalthemelayout->addWidget(m_rightwidget);
    this->setLayout(globalthemelayout);
}

/**
 * @brief 初始化预览小部件
 *
 * 创建一个新的QWidget作为预览小部件，并设置其最小大小。
 * 设置预览小部件的背景角色为QPalette::Window。
 * 设置预览小部件的背景颜色为"#F5F5F5"。
 * 设置预览小部件自动填充背景。
 * 创建一个新的GrubImageWidget作为预览图像小部件。
 * 创建一个新的QVBoxLayout作为预览小部件的布局。
 * 添加一个空项。
 * 设置预览小部件的布局。
 */
void GrubThemeWidget::initPreviewWidget()
{
    m_previewwidget = new QWidget(this);
    m_previewwidget->setMinimumSize(495,620);

    m_previewwidget->setBackgroundRole(QPalette::Window);

    QPalette palette;
    QColor bgColor("#F5F5F5");
    palette.setColor(QPalette::Window, bgColor);
    m_previewwidget->setPalette(palette);
    m_previewwidget->setAutoFillBackground(true);

    m_preview = new GrubImageWidget(m_previewwidget);

    m_previewlayout = new QVBoxLayout(m_previewwidget);
    addspaceritem();
    m_previewwidget->setLayout(m_previewlayout);
}

/**
 * @brief 初始化右侧小部件
 *
 * 创建一个新的 QWidget，并将其设置为当前对象的成员变量 m_rightwidget。
 * 设置最小大小为 400x620，最大宽度为 400。
 * 设置背景角色为 QPalette::Base。
 * 调用 initEditWidget() 函数初始化编辑小部件。
 */
void GrubThemeWidget::initRightWidget()
{
    m_rightwidget = new QWidget(this);
    m_rightwidget->setMinimumSize(400,620);
    m_rightwidget->setMaximumWidth(400);
    m_rightwidget->setBackgroundRole(QPalette::Base);
    initEditWidget();
}

/**
 * @brief 初始化编辑小部件
 *
 * 创建一个新的 QWidget，设置固定高度，并添加垂直布局。
 * 创建两个 QLabel 和一个 QPushButton，并将它们添加到布局中。
 * 创建 QHBoxLayout，将 QPushButton 和 QLabel 添加到布局中。
 * 连接 QPushButton 的 clicked 信号，当点击按钮时，打开文件对话框选择图片文件。
 * 如果选择的文件合法，则更新背景预览并设置 QPushButton 的图标。
 * 发出 newGrubFilePath 信号以更新新的 Grub 文件路径。
 */
void GrubThemeWidget::initEditWidget()
{
    m_grubwidget = new QWidget(m_rightwidget);
    m_grubwidget->setFixedHeight(150);
    m_grubwidget->setFixedHeight(183);
    QVBoxLayout *layout = new QVBoxLayout();

    QLabel *tipLabel = new QLabel("Grub背景");
    QWidget *grubWidget = new QWidget();
    QHBoxLayout *widgetLayout = new QHBoxLayout();

    QPushButton *showBtn = new QPushButton();
    showBtn->setFixedSize(160,100);

    QLabel *tipLabel1 = new QLabel();
    tipLabel1->setText("<html>尺寸：3840*2160<br>大小：不超过 10 MB<br>格式：PNG</html>");

    connect(showBtn, &QPushButton::clicked, this, [=]() {
        // 弹出文件选择对话框，选择图片文件
        QString newFilePath = QFileDialog::getOpenFileName(this, tr("Select picture file"), "", tr("Picture file (*.png *.jpg)"));

        // 如果选择的文件路径不为空
        if (!newFilePath.isEmpty()) {
            // 判断选择的文件是否是合法的壁纸文件
            if(FileCheck::isLegalWallPaperFile(newFilePath,"wallpaper")){
                // 更新背景预览
                m_preview->updateBackground(newFilePath);
                // 加载图片文件并创建一个QPixmap对象
                QPixmap pixmap(newFilePath);
                // 设置按钮的图标为加载的图片文件对应的图标，并设置图标大小为按钮的大小
                showBtn->setIcon(QIcon(pixmap));
                showBtn->setIconSize(showBtn->size());
                // 发出新的Grub文件路径信号
                emit newGrubFilePath(newFilePath);
                g_themeChange = true;
            }
        } else {
            qDebug() << "Selected file path:" << newFilePath;
        }
    });

    widgetLayout->addWidget(showBtn);
    widgetLayout->addWidget(tipLabel1);
    grubWidget->setLayout(widgetLayout);

    layout->addWidget(tipLabel);
    layout->addWidget(grubWidget);
    m_grubwidget->setLayout(layout);
}

void GrubThemeWidget::addspaceritem()
{
    QSpacerItem *spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding);
    QSpacerItem *spacer2 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding);
    spacer->changeSize(m_previewwidget->width(), 155, QSizePolicy::Maximum, QSizePolicy::Maximum);
    spacer2->changeSize(m_previewwidget->width(), 159, QSizePolicy::Maximum, QSizePolicy::Maximum);

    m_previewlayout->addItem(spacer);
    m_previewlayout->addWidget(m_preview);
    m_previewlayout->addItem(spacer2);
}



